# tres/Scripts/thartable.cl
# June 6, 2008
# By Doug Mink

# THARTABLE - Tabulate pixel shifts in COMP spectra by aperture size, fiber(s), and binning

procedure thartable ()

string	apfib=""	{prompt="Select fiber (s m l) binning (b) ap (1 2 3)"}
string	fibcode=""	{prompt="FIBSIZE value to match: null=all s=1.4 m=2.3 l=3.2"}
int	binning=0	{prompt="BINNING value to match: null=any 0=any 1 2"}
string	apertures="1"	{prompt="Aperture to use: 1 2 3"}
string	outfile=""	{prompt="Returned name of output table file"}
bool	overwrite=no	{prompt="Overwrite existing list files (yes or no)"}
bool	deltemp=yes	{prompt="Delete temporary files (yes or no)"}
bool    verbose=yes	{prompt="Print process progress (yes or no)"}
bool    debug=no	{prompt="Print debugging info (yes or no)"}

begin

    string ims, ap, aps, afx, exp, listfile, filetemp, fname
    string dline1, dline2, imin, fibsize, bincode, refthar
    bool vb, ow, db, dt, first
    int maxlfile, filebin, nfile

    vb = verbose
    db = debug
    dt = deltemp
    ow = overwrite

    afx = apfib
    if (afx != "") {
	trsdecode (afx)
	bincode = trsdecode.bincode
	binning = trsdecode.binning
	fibcode = trsdecode.fibcode
	fibsize = trsdecode.fibsize
	ap = trsdecode.apertures
	if (ap == "")
	    aps = "*"
	else
	    aps = ap
	}
    else {
	ap = apertures
	if (ap == "")
	    aps = "*"
	if (binning == 2)
	    bincode = "b"
	else
	    bincode = ""
	afx = fibcode // bincode // ap
	}

# Make list of ThAr files with this aperture
    listfile = "comp" // ap // ".list"
    if (!access (listfile) || overwrite) {
	if (access (listfile)) {
	    if (db)
		print ("THARTABLE: Deleting ",listfile)
	    delete (listfile)
	    }
	filetemp = "*" // "COMP_fib" // aps // ".ec.fits"
	if (db)
	    print ("THARTABLE: Listing ",filetemp," files to ",listfile)
	files (filetemp, >listfile)
	}

# Make file column headings
    fname = "filename"
    maxlfile = 22
    for (i = 9; i <= maxlfile; i = i + 1) {
	fname = fname // " "
	}
    dline1 = fname // "	refthar        	mjdmid         	shift    "
    fname = "-"
    for (i = 2; i <= maxlfile; i = i + 1) {
	fname = fname // "-"
	}
    dline2 = fname // "	---------------	---------------	--------"

# Make table of MJD and pixel shift for ThAr files with specified binning and aperture size
    outfile = "comp" // afx // ".shift"
    if (access (outfile)) {
	if (overwrite)
	    delete (outfile)
	else {
	    if (db)
		print ("THARTAB: ", outfile, " already exists.")
	    return
	    }
	}
    if (debug)
	print ("THARTABLE: Writing to ",outfile)
    first = yes
    nfile = 0
    list = listfile
    while (fscan (list, imin) != EOF) {
	tfib1 (imin)
	apfib = tfib1.apfib
	if (db)
	    print (imin," apfib = ",tfib1.apfib," compared to ",afx)
	if (tfib1.apfib != afx)
	    next
	imgetp.default = "0"
	imgetp (imin, "TIDSHIFT")
	cround (imgetp.value, "%9.6f")
	imgetp.default = "0"
	imgetp (imin, "TIDREF")
	refthar = imgetp.value
	if (first) {
	    print (dline1, >outfile)
	    print (dline2, >>outfile)
	    if (vb) {
		print (dline1)
		print (dline2)
		}
	    first = no
	    }
	nfile = nfile + 1
	print (imin,"	", refthar, "	", tfib1.mjdmid, "	", cround.value, >>outfile)
	if (vb)
	    print (imin,"	", refthar, "	", tfib1.mjdmid, "	", cround.value)
	}
    if (nfile < 1)
	outfile = ""

    if (dt)
	delete (listfile)

end

# May 20 2008	New task
# May 22 2008	Add name of reference spectrum for shift to table
# May 30 2008	Get apfib code and MJD mid-time for spectra using TRSFIB
# Jun  6 2008	Call TRSFIB1 instead of TRSFIB
