# TIDREF -- Find reference calibration spectrum for a TRES spectrum
# By Doug Mink, SAO Telescope Data Center
# July 16, 2008

procedure tidref (spectrum)

file	spectrum=""	{prompt="Spectrum for which to find reference"}

file	refroot="comp"	{prompt="Root filename for dispersion reference spectrum"}
file	refdir="tresdata$"	{prompt="Directory for reference spectra"}
file	reffile=""	{prompt="Dispersion reference file (returned)"}
file	refpath=""	{prompt="Dispersion reference file path (returned)"}
string	mjdmid=""	{prompt="MJD midtime of this ThAr exposure (returned)"}
bool	verbose=yes	{prompt="Log progress (y or n)"}
bool	debug=yes	{prompt="Print debugging information (y or n)"}

begin
	bool vb, db
	string rroot, binning, apfib
	file rdir, spec
	string fiber

	spec = spectrum
	rdir = refdir
	fixdir (rdir)
	rdir = fixdir.newdir
#	if (rdir == "") {
#	    rdir = "tresdata$"
#	    }
	rroot = refroot
	if (rroot == "") {
	    rroot = "comp"
	    }
	vb = verbose
	db = debug

# Get fiber, binning, and aperture size for this spectrum
	tfib1 (spec, specdir="", verbose=db, debug=db)
	fiber = tfib1.fiber
	binning = tfib1.binning
	apfib = tfib1.apfib
	round (tfib1.mjdmid, "%11.5f")
	mjdmid = round.value

# Get file name for fiber reference image
	reffile = rroot // apfib // ".ec.fits"
	refpath = rdir // reffile
	if (!access (refpath)) {
	    if (db)
		print ("TIDREF: ", refpath, " does not exist")
	    reffile = rroot // binning // fiber // ".ec.fits"
	    refpath = rdir // reffile
	    if (!access (refpath)) {
		if (db)
		    print ("TIDREF: ", refpath, " does not exist")
		reffile = rroot // binning // "1.ec.fits"
		refpath = rdir // reffile
		if (!access (refpath)) {
			reffile = ""
		    if (vb)
			print ("TIDREF: ", refpath, " does not exist")
		    return
		    }
		}
	    }
	if (db)
	    print ("TIDREF: Undispersed calibration reference file is ",refpath)
	else if (vb)
	    print (refpath)

	return
end

# Jan 30 2008	New task
# Feb 13 2008	Set specdir="" in call to trsfib
# Mar 11 2008	Fixed xcsao; no need for separate p.ec.fits files
# Jun  6 2008	Call TRSFIB1 instead of TRSFIB
# July 16 2008	Convert mjdmid from TRSFIB1 to a string and return it
