# File tres/Scripts/trsgroup.cl
# September 9, 2008
# By Doug Mink

# TRSGROUP: Make lists of files to process

procedure trsgroup (images)

string	images="1-300"	{prompt="Files to process"}

string	datedir=""	{prompt="Date for images"}
file	instdir=""	{prompt="Directory containing date directories"}
file	imagedir=""	{prompt="Directory containing images (overrides above)"}
file	datafile="data.db"	{prompt="File for observation database"}
file	sortfile="datasort.db"	{prompt="File for sorted observation database"}
file	biaslist="bias.list"	{prompt="File listing groups of bias images"}
file	darklist="dark.list"	{prompt="File listing groups of dark images"}
file	flatlist="flat.list"	{prompt="File listing groups of flat fields"}
file	complist="comp.list"	{prompt="File listing groups of ThAr exposures"}
file	objlist="obj.list"	{prompt="File listing groups of observations"}
bool	verbose=yes	{prompt="Summarize results (yes or no)"}
bool	debug=no	{prompt="Print file sorting instead of writing files (yes or no)"}

struct *sortlist

begin
    file listname
    file imf, compname
    string imt, obj, mjd, apfib, fibkey, obj0, apfib0
    int lfile, ifile, ibias, idark, iexp, ngroups, iobj, icomp, iflat
    real exp, comp, exp0, comp0
    bool savespec

    biaslist = trsproc.biaslist
    darklist = trsproc.darklist
    flatlist = trsproc.flatlist
    complist = trsproc.complist
    objlist = trsproc.objlist

# List all of the data files for this night
    if (access (datafile)) {
	print ("TRSGROUP: Reading database from existing file ",datafile)
	}
    else {
	trslist (images, datedir=datedir, instdir=instdir, imagedir=imagedir,
		 imagetype="", object="",apfib="",aperture="",fibkey="",
		 binning="",fibsize="",exptime="",outlist=datafile,
		 addfib=no,deltemp=yes,overwrite=no,verbose=yes,debug=no)
	if (access (sortfile))
	    delete (sortfile)
	}

# Sort data files by object name, fiber size/binning/numbers, exposure, time
    if (access (sortfile)) {
	print ("TRSGROUP: Reading sorted database from existing file ",sortfile)
	}
    else {
	args = "<" // datafile // " -k7 -k5 -k3 >" // sortfile
	trssort (args)
	}

# Write list files and master list of list files for processing
    obj0 = ""
    apfib0 =""
    exp0 = 0.0
    ifile = 0
    lfile = 0
    ibias = 0
    idark = 0
    iflat = 0
    icomp = 0
    iobj = 0
    savespec = no
    listname = ""
    sortlist = sortfile
    while (fscan (sortlist,imf,imt,obj,mjd,apfib,fibkey,exp,comp) != EOF) {
#	print ("TRSGROUP: File ", imf)

# Start a new configuration file if the configuration changes
	if (obj != obj0 || apfib != apfib0 || exp != exp0) {
	    if (debug && listname != "") {
		print ("TRSGROUP: ",lfile," files in list ", listname)
		}
	    iexp = int (exp)
	    listname = obj // apfib // "x" //iexp // ".list"
	    compname = obj // apfib // ".list"
	    savespec = no

# Check for existence of file for this object and configuration
	    if (!debug && access (listname)) {
		if (verbose)
		    print ("TRSGROUP: Object list file ",listname," already exists")
		next
		}

# Add bias file names to list
	    if (biaslist != "" && obj=="BIAS") {
		savespec = yes
		if (debug) {
		    if (ibias == 0)
			print (listname, " > ", biaslist)
		    else
			print (listname, " >> ", biaslist)
		    }
		else {
		    if (ibias == 0)
			print (listname, >biaslist)
		    else
			print (listname, >>biaslist)
		    }
		ibias = ibias + 1
		}

# Add dark file names to list
	    if (darklist != "" && obj=="DARK") {
		savespec = yes
		if (debug) {
		    if (idark == 0)
			print (listname, " > ", darklist)
		    else
			print (listname, " >> ", darklist)
		    }
		else {
		    if (idark == 0)
			print (listname, >darklist)
		    else
			print (listname, >>darklist)
		    }
		idark = idark + 1
		}

# Add ThAr file names to list
	    else if (complist != "" && obj == "COMP") {
		savespec = yes
		if (debug) {
		    if (icomp == 0)
			print (listname, " > ", complist)
		    else
			print (listname, " >> ", complist)
		    }
		else {
		    if (icomp == 0)
			print (listname, >complist)
		    else
			print (listname, >>complist)
		    }
		icomp = icomp + 1
		}

# Add flat field file names to list
	    else if (flatlist != "" && obj == "FLAT") {
		savespec = yes
		if (debug) {
		    if (iflat == 0)
			print (listname, " > ", flatlist)
		    else
			print (listname, " >> ", flatlist)
		    }
		else {
		    if (iflat == 0)
			print (listname, >flatlist)
		    else
			print (listname, >>flatlist)
		    }
		iflat = iflat + 1
		}

# Add object configuration file names to list
	    else if (objlist != "") {
		savespec = yes
		if (debug) {
		    if (iobj == 0)
			print (listname, " > ", objlist)
		    else
			print (listname, " >> ", objlist)
		    }
		else {
		    if (iobj == 0)
			print (listname, >objlist)
		    else
			print (listname, >>objlist)
		    }
		iobj = iobj + 1
		}

	    else if (verbose) {
		print ("TRSGROUP: Not saving ",listname)
		}

# Save this image file name to the new configuration list file
	    if (debug)
		print (imf, " > ", listname)
	    else if (savespec)
		print (imf, >listname)
	    lfile = 1

# Add ThAr file names to exposure-free list, too
	    if (obj == "COMP") {
		if (debug) {
		    if (access (compname))
			print (imf, " >> ", compname)
		    else
			print (imf, " > ", compname)
		    }

		else {
		    if (access (compname))
			print (imf, >>compname)
		    else
			print (imf, >compname)
		    }
		}
	    }

# Save this image file name to the current configuration list file
	else if (savespec) {
	    if (debug)
		print (imf, " >> ",listname)
	    else
		print (imf, >>listname)
	    lfile = lfile + 1

# Add ThAr file names to exposure-free lists, too
	    if (obj == "COMP") {
		if (debug)
		    print (imf, " >> ", compname)
		else
		    print (imf, >>compname)
		}
	    }
	ifile = ifile + 1
	exp0 = exp
	apfib0 = apfib
	obj0 = obj
	}

# Summarize the sorting process
    ngroups = iobj + ibias + idark + iflat + icomp
    if (verbose)
	print ("TRSGROUP: ", ifile, "files split into ", ngroups, " groups")

    sortlist = ""
end
# Jul 16 2008	New task
# Jul 30 2008	Make additional lists of COMP files by configuration only
# Aug 15 2008	Separate dark and bias image lists; list names are in TRSPROC
# Sep  9 2008	Fix bug so that all ThAr files are saved to exp-free lists
