# File hectospec/Src/t_gettime.x
# May 16, 2008
# By Doug Mink, Harvard-Smithsonian Center for Astrophysics

# T_GETTIME -- return current date and time as a string parameter

include <time.h>

define  SZ_WEEKDAY              3
define  SZ_MONTH                3
define	MAXCH	32

int procedure t_gettime()

char	dstring[MAXCH], tstring[MAXCH], fstring[MAXCH], dweek[8]
char	datadir[MAXCH]
bool	usesec, verbose, clgetb()
double	mjd, jd
long	iraf_time, data_time, clktime()
int	ihr, imin, isec, iday, imon, iyr, iwday, dday
int	tm[LEN_TMSTRUCT]
int	clgeti()
double	clgetd()
string	weekday "SunMonTueWedThuFriSat"
string	month   "JanFebMarAprMayJunJulAugSepOctNovDec"

begin

	mjd = clgetd ("intime")
	if (mjd > 100000.d0) {
	    jd = mjd
	    mjd = 0.d0
	    }
	iraf_time = 0

# Convert Modified Julian Day to IRAF seconds since 1980-01-01T0:00
	if (mjd != 0.d0)
	    iraf_time = int (((mjd - 44239.0d0) * 8.64d4) + 0.5d0)

# Convert Julian Day to IRAF seconds since 1980-01-01T0:00
	else if (jd != 0.d0)
	    iraf_time = int (((jd - 2444239.5d0) * 8.64d4) + 0.5d0)

# Get current IRAF time (seconds since 1980-01-01T0:00)
	else {
	    iraf_time = clktime (iraf_time)

# Check for date difference only if starting from current time
	    dday = clgeti ("datediff")
	    if (dday != 0) {
		iraf_time = iraf_time + (dday * 86400)
		}
	    }
#	call printf ("GETTIME: IRAF Time is %d seconds past 1980-01-01T00:00\n")
#	    call pargi (iraf_time)

# Break up seconds into something usable
	call brktime (iraf_time, tm)
	ihr = TM_HOUR(tm)
	imin = TM_MIN(tm)
	isec = TM_SEC(tm)
	iday = TM_MDAY(tm)
	imon = TM_MONTH(tm)
	iyr = TM_YEAR(tm)
	iwday = TM_WDAY(tm)

	usesec = clgetb ("usesec")
	if (usesec) {
	    call sprintf (tstring, MAXCH, "%02d:%02d:%02d")
		call pargi (ihr)
		call pargi (imin)
		call pargi (isec)
	    }
	else {
	    if (isec > 30)
		imin = imin + 1
	    if (imin > 59) {
		ihr = ihr + 1
		imin = imin - 60
		}
	    if (ihr > 23) {
		iday = iday + 1
		ihr = ihr - 24
		}
	    call sprintf (tstring, MAXCH, "%02d:%02d")
		call pargi (TM_HOUR(tm))
		call pargi (TM_MIN(tm))
	    }
	call clpstr ("time", tstring)

	call sprintf (fstring, MAXCH, "%04d-%02d-%02dT%s")
	    call pargi (iyr)
	    call pargi (imon)
	    call pargi (iday)
	    call pargstr (tstring)
	call clpstr ("fitsdate", fstring)

	call sprintf (dstring, MAXCH, "%04d-%3.3s-%02d")
	    call pargi (iyr)
	    call pargstr (month [(imon - 1) * SZ_MONTH + 1])
	    call pargi (iday)
	call clpstr ("date", dstring)

	call sprintf (dweek, 8, "%3.3s")
	    call pargstr (weekday[((iwday-1)*3)+1])
	call clpstr ("dweek", dweek)

# Get Mt. Hopkins data directory for this time
	data_time = iraf_time - 43200
	call brktime (data_time, tm)
	iday = TM_MDAY(tm)
	imon = TM_MONTH(tm)
	iyr = TM_YEAR(tm)
	call sprintf (datadir, MAXCH, "%04d.%02d%02d")
	    call pargi (iyr)
	    call pargi (imon)
	    call pargi (iday)
	call clpstr ("datadir", datadir)

	verbose = clgetb ("verbose")
	if (verbose) {
	    call printf ("%s %s %s: %s\n")
		call pargstr (dweek)
		call pargstr (dstring)
		call pargstr (tstring)
		call pargstr (datadir)
	    }
	return
end

# May 21 2004	New task
# Feb  2 2008	Add dweek parameter for day of week
# Feb 13 2008	Add datediff and datadir parameters
# May 16 2008	Add intime = input Julian Day or Modified Julian Day
